@@ -5,7 +5,7 @@
 @license: LGPLv3+
 '''
 
-__version__ = '0.3.0'
+__version__ = '0.5.1'
 
 import gtk, gobject
 import gtkdialogs
@@ -18,7 +18,7 @@ import re
 import urllib2
 
 try: import i18n
-except: import gettext.gettext as _
+except: from gettext import gettext as _
 
 def get_doctitle(html):
     title = ''
@@ -26,6 +26,22 @@ def get_doctitle(html):
     return title
 
 
+def proc_webkit_color(*webviews):
+    ## 设置样式，让 WebKit 背景色使用 Gtk 颜色
+    style = webviews[0].get_style()
+    html_bg_color = str(style.base[gtk.STATE_NORMAL])
+    html_fg_color = str(style.text[gtk.STATE_NORMAL])
+    user_stylesheet = ''' html {
+        background-color: %s;
+        color: %s;\n}''' % (html_bg_color, html_fg_color)
+    user_stylesheet_file = config.user_stylesheet_file
+    file(user_stylesheet_file, 'w').write(user_stylesheet)
+    user_stylesheet_uri = 'file://' + user_stylesheet_file
+    for webview in webviews:
+        settings = webview.get_settings()
+        settings.set_property('user-stylesheet-uri', user_stylesheet_uri)
+    pass
+
 def menu_find_with_stock(menu, stock):
     # 查找菜单中对应 stock 的菜单项位置
     n = 0
@@ -47,7 +63,7 @@ Title = _("GWrite")
 #mdi_mode = 1
 
 class MainWindow:
-    def __init__(self, editfile='', create = True, accel_group = None, tooltips = None):
+    def __init__(self, editfile='', create = True, accel_group = None):
         self.editfile = editfile
         ## 考虑已经打开文档的情况
         if editfile:
@@ -71,22 +87,22 @@ class MainWindow:
             self.accel_group = gtk.AccelGroup()
         else:
             self.accel_group = accel_group
-        if tooltips is None:
-            self.tooltips = gtk.Tooltips()
-        else:
-            self.tooltips = tooltips
-        self.tooltips.enable()
         if create:
             self.window = gtk.Window()
             gtk.window_set_default_icon_name("gtk-dnd")
             self.window.set_icon_name("gtk-dnd")
-            self.window.set_default_size(750, 550)
+            self.window.set_default_size(780, 550)
             self.window.set_title(Title)
             if editfile: self.window.set_title(os.path.basename(self.editfile) + ' - ' + Title) 
             self.window.add_accel_group(self.accel_group)
             self.window.show()
             self.window.connect("delete_event", self.on_close)
 
+        ## 用 Alt-1, Alt-2... 来切换标签页，gtk.gdk.MOD1_MASK 是 Alt
+        for k in range(1, 10):
+            self.accel_group.connect_group(gtk.gdk.keyval_from_name(str(k)), gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE, self.on_accel_connect_group)
+            pass
+
         self.vbox1 = gtk.VBox(False, 0)
         self.vbox1.show()
 
@@ -255,7 +271,7 @@ class MainWindow:
         menuitem_paste_unformatted.show()
         menuitem_paste_unformatted.connect("activate", self.do_paste_unformatted)
         menuitem_paste_unformatted.add_accelerator("activate", 
-                self.accel_group, gtk.gdk.keyval_from_name("period"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+                self.accel_group, gtk.gdk.keyval_from_name("v"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
         menu_edit.append(menuitem_paste_unformatted)
 
         menuitem_delete = gtk.ImageMenuItem("gtk-delete")
@@ -372,7 +388,7 @@ class MainWindow:
 
         menu_view.append(menuitem_separator10)
 
-        menuitem_view_source = gtk.ImageMenuItem(_("So_urce"))
+        menuitem_view_source = gtk.ImageMenuItem(_("So_urce/Visual"))
         menuitem_view_source.show()
         menuitem_view_source.connect("activate", self.view_sourceview)
         menuitem_view_source.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("u"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
@@ -402,6 +418,7 @@ class MainWindow:
         menuitem_link = gtk.ImageMenuItem(_("_Link"))
         menuitem_link.show()
         menuitem_link.connect("activate", self.do_createlink)
+        menuitem_link.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("k"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_link', gtk.ICON_SIZE_MENU)
         menuitem_link.set_image(img)
@@ -467,6 +484,7 @@ class MainWindow:
         menuitem_normal = gtk.ImageMenuItem(_("_Normal"))
         menuitem_normal.show()
         menuitem_normal.connect("activate", self.do_formatblock_p)
+        menuitem_normal.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("0"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_insert_section', gtk.ICON_SIZE_MENU)
         menuitem_normal.set_image(img)
@@ -480,6 +498,7 @@ class MainWindow:
         menuitem_heading_1 = gtk.ImageMenuItem(_("Heading _1"))
         menuitem_heading_1.show()
         menuitem_heading_1.connect("activate", self.do_formatblock_h1)
+        menuitem_heading_1.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("1"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_insert-header', gtk.ICON_SIZE_MENU)
         menuitem_heading_1.set_image(img)
@@ -488,6 +507,7 @@ class MainWindow:
         menuitem_heading_2 = gtk.ImageMenuItem(_("Heading _2"))
         menuitem_heading_2.show()
         menuitem_heading_2.connect("activate", self.do_formatblock_h2)
+        menuitem_heading_2.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("2"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_line-spacing-2', gtk.ICON_SIZE_MENU)
         menuitem_heading_2.set_image(img)
@@ -496,6 +516,7 @@ class MainWindow:
         menuitem_heading_3 = gtk.ImageMenuItem(_("Heading _3"))
         menuitem_heading_3.show()
         menuitem_heading_3.connect("activate", self.do_formatblock_h3)
+        menuitem_heading_3.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("3"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_line-spacing-1', gtk.ICON_SIZE_MENU)
         menuitem_heading_3.set_image(img)
@@ -504,6 +525,7 @@ class MainWindow:
         menuitem_heading_4 = gtk.ImageMenuItem(_("Heading _4"))
         menuitem_heading_4.show()
         menuitem_heading_4.connect("activate", self.do_formatblock_h4)
+        menuitem_heading_4.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("4"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_line-spacing-1.5', gtk.ICON_SIZE_MENU)
         menuitem_heading_4.set_image(img)
@@ -512,6 +534,7 @@ class MainWindow:
         menuitem_heading_5 = gtk.ImageMenuItem(_("Heading _5"))
         menuitem_heading_5.show()
         menuitem_heading_5.connect("activate", self.do_formatblock_h5)
+        menuitem_heading_5.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("5"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_list_enum-off', gtk.ICON_SIZE_MENU)
         menuitem_heading_5.set_image(img)
@@ -520,6 +543,7 @@ class MainWindow:
         menuitem_heading_6 = gtk.ImageMenuItem(_("Heading _6"))
         menuitem_heading_6.show()
         menuitem_heading_6.connect("activate", self.do_formatblock_h6)
+        menuitem_heading_6.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("6"), gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_list_enum-off', gtk.ICON_SIZE_MENU)
         menuitem_heading_6.set_image(img)
@@ -551,6 +575,14 @@ class MainWindow:
 
         menu_style.append(menuitem_separator6)
 
+        div1 = gtk.ImageMenuItem(_("Di_v"))
+        div1.show()
+        div1.connect("activate", self.do_formatblock_div)
+
+        img = gtk.image_new_from_icon_name('stock_tools-hyphenation', gtk.ICON_SIZE_MENU)
+        div1.set_image(img)
+        menu_style.append(div1)
+
         address1 = gtk.ImageMenuItem(_("A_ddress"))
         address1.show()
         address1.connect("activate", self.do_formatblock_address)
@@ -578,6 +610,7 @@ class MainWindow:
         menuitem_formatblock_pre = gtk.ImageMenuItem(_("_Preformat"))
         menuitem_formatblock_pre.show()
         menuitem_formatblock_pre.connect("activate", self.do_formatblock_pre)
+        menuitem_formatblock_pre.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("t"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_text-quickedit', gtk.ICON_SIZE_MENU)
         menuitem_formatblock_pre.set_image(img)
@@ -600,18 +633,20 @@ class MainWindow:
 
         menu_format.append(menuitem_bold)
 
-        menuitem_underline = gtk.ImageMenuItem("gtk-underline")
-        menuitem_underline.show()
-        menuitem_underline.connect("activate", self.do_underline)
-
-        menu_format.append(menuitem_underline)
-
         menuitem_italic = gtk.ImageMenuItem("gtk-italic")
         menuitem_italic.show()
         menuitem_italic.connect("activate", self.do_italic)
+        menuitem_italic.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("i"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
         menu_format.append(menuitem_italic)
 
+        menuitem_underline = gtk.ImageMenuItem("gtk-underline")
+        menuitem_underline.show()
+        menuitem_underline.connect("activate", self.do_underline)
+        menuitem_underline.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("u"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+
+        menu_format.append(menuitem_underline)
+
         menuitem_strikethrough = gtk.ImageMenuItem("gtk-strikethrough")
         menuitem_strikethrough.show()
         menuitem_strikethrough.connect("activate", self.do_strikethrough)
@@ -630,7 +665,17 @@ class MainWindow:
         ## 字体列表菜单 #########################################
         self.fontname_menu = gtk.Menu()
         self.fontname_menu.append(gtk.TearoffMenuItem())
-        fontnames = list(reversed(sorted(( familie.get_name() for familie in gtk.Label().get_pango_context().list_families() ))))
+        fontnames = sorted(( familie.get_name() for familie in gtk.Label().get_pango_context().list_families() ))
+        ## 调整字体列表顺序，将中文字体提至前列
+        for fontname in fontnames:
+            try:
+                fontname.decode('ascii')
+                pass
+            except:
+                fontnames.remove(fontname)
+                fontnames.insert(0, fontname)
+                pass
+            pass
         for fontname in ['Serif', 'Sans', 'Sans-serif', 'Monospace', ''] + fontnames:
             if fontname:
                 menu = gtk.MenuItem(fontname)
@@ -730,6 +775,7 @@ class MainWindow:
         menuitem_clearformat.set_image(img)
         menuitem_clearformat.show()
         menuitem_clearformat.connect("activate", self.do_removeformat)
+        menuitem_clearformat.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("backslash"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
         menu_format.append(menuitem_clearformat)
 
@@ -741,21 +787,31 @@ class MainWindow:
         menuitem_justifyleft = gtk.ImageMenuItem("gtk-justify-left")
         menuitem_justifyleft.show()
         menuitem_justifyleft.connect("activate", self.do_justifyleft)
+        menuitem_justifyleft.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("l"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
 
         menu_format.append(menuitem_justifyleft)
 
         menuitem_justifycenter = gtk.ImageMenuItem("gtk-justify-center")
         menuitem_justifycenter.show()
         menuitem_justifycenter.connect("activate", self.do_justifycenter)
+        menuitem_justifycenter.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("e"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
 
         menu_format.append(menuitem_justifycenter)
 
         menuitem_justifyright = gtk.ImageMenuItem("gtk-justify-right")
         menuitem_justifyright.show()
         menuitem_justifyright.connect("activate", self.do_justifyright)
+        menuitem_justifyright.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("r"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
 
         menu_format.append(menuitem_justifyright)
 
+        menuitem_justifyfull = gtk.ImageMenuItem("gtk-justify-fill")
+        menuitem_justifyfull.show()
+        menuitem_justifyfull.connect("activate", self.do_justifyfull)
+        menuitem_justifyfull.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("j"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
+
+        menu_format.append(menuitem_justifyfull)
+
         self.separator11 = gtk.MenuItem()
         self.separator11.show()
 
@@ -778,22 +834,24 @@ class MainWindow:
 
         menu_format.append(self.separator16)
 
-        menuitem_subscript = gtk.ImageMenuItem(_("Subs_cript"))
-        menuitem_subscript.show()
-        menuitem_subscript.connect("activate", self.do_subscript)
-
-        img = gtk.image_new_from_icon_name('stock_subscript', gtk.ICON_SIZE_MENU)
-        menuitem_subscript.set_image(img)
-        menu_format.append(menuitem_subscript)
-
         menuitem_superscript = gtk.ImageMenuItem(_("Su_perscript"))
         menuitem_superscript.show()
         menuitem_superscript.connect("activate", self.do_superscript)
+        menuitem_superscript.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("period"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
         img = gtk.image_new_from_icon_name('stock_superscript', gtk.ICON_SIZE_MENU)
         menuitem_superscript.set_image(img)
         menu_format.append(menuitem_superscript)
 
+        menuitem_subscript = gtk.ImageMenuItem(_("Subs_cript"))
+        menuitem_subscript.show()
+        menuitem_subscript.connect("activate", self.do_subscript)
+        menuitem_subscript.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("comma"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+
+        img = gtk.image_new_from_icon_name('stock_subscript', gtk.ICON_SIZE_MENU)
+        menuitem_subscript.set_image(img)
+        menu_format.append(menuitem_subscript)
+
         menuitem_format.set_submenu(menu_format)
 
         menubar1.append(menuitem_format)
@@ -809,6 +867,7 @@ class MainWindow:
         menuitem_word_count.set_image(img)
         menuitem_word_count.show()
         menuitem_word_count.connect("activate", self.on_word_counts)
+        menuitem_word_count.add_accelerator("activate", self.accel_group, gtk.gdk.keyval_from_name("c"), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
 
         menu_tools.append(menuitem_word_count)
 
@@ -843,14 +902,14 @@ class MainWindow:
         self.toolbar1.show()
 
         toolbutton_new = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_new, _("New"))
+        toolbutton_new.set_tooltip_text(_("New"))
         toolbutton_new.show()
         toolbutton_new.set_stock_id(gtk.STOCK_NEW)
         toolbutton_new.connect("clicked", self.on_new)
         self.toolbar1.add(toolbutton_new)
 
         toolbutton_open = gtk.MenuToolButton(gtk.STOCK_OPEN)
-        self.tooltips.set_tip(toolbutton_open, _("Open"))
+        toolbutton_open.set_tooltip_text(_("Open"))
         toolbutton_open.show()
         #toolbutton_open.set_stock_id(gtk.STOCK_OPEN)
         toolbutton_open.connect("clicked", self.on_open)
@@ -858,7 +917,7 @@ class MainWindow:
         self.toolbar1.add(toolbutton_open)
 
         toolbutton_save = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_save, _("Save"))
+        toolbutton_save.set_tooltip_text(_("Save"))
         toolbutton_save.show()
         toolbutton_save.set_stock_id(gtk.STOCK_SAVE)
         toolbutton_save.connect("clicked", self.on_save)
@@ -869,14 +928,14 @@ class MainWindow:
         self.toolbar1.add(separatortoolitem1)
 
         toolbutton_undo = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_undo, _("Undo"))
+        toolbutton_undo.set_tooltip_text(_("Undo"))
         toolbutton_undo.show()
         toolbutton_undo.set_stock_id(gtk.STOCK_UNDO)
         toolbutton_undo.connect("clicked", self.do_undo)
         self.toolbar1.add(toolbutton_undo)
 
         toolbutton_redo = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_redo, _("Redo"))
+        toolbutton_redo.set_tooltip_text(_("Redo"))
         toolbutton_redo.show()
         toolbutton_redo.set_stock_id(gtk.STOCK_REDO)
         toolbutton_redo.connect("clicked", self.do_redo)
@@ -887,21 +946,21 @@ class MainWindow:
         self.toolbar1.add(separatortoolitem3)
 
         toolbutton_cut = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_cut, _("Cut"))
+        toolbutton_cut.set_tooltip_text(_("Cut"))
         toolbutton_cut.show()
         toolbutton_cut.set_stock_id(gtk.STOCK_CUT)
         toolbutton_cut.connect("clicked", self.do_cut)
         self.toolbar1.add(toolbutton_cut)
 
         toolbutton_copy = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_copy, _("Copy"))
+        toolbutton_copy.set_tooltip_text(_("Copy"))
         toolbutton_copy.show()
         toolbutton_copy.set_stock_id(gtk.STOCK_COPY)
         toolbutton_copy.connect("clicked", self.do_copy)
         self.toolbar1.add(toolbutton_copy)
 
         toolbutton_paste = gtk.ToolButton()
-        self.tooltips.set_tip(toolbutton_paste, _("Paste"))
+        toolbutton_paste.set_tooltip_text(_("Paste"))
         toolbutton_paste.show()
         toolbutton_paste.set_stock_id(gtk.STOCK_PASTE)
         toolbutton_paste.connect("clicked", self.do_paste)
@@ -915,7 +974,7 @@ class MainWindow:
         label1 = gtk.Label("")
         label1.set_markup("<b>P</b>")
         button1 = gtk.ToolButton(label1, _("Paragraph"))
-        self.tooltips.set_tip(button1, _("Paragraph"))
+        button1.set_tooltip_text(_("Paragraph"))
         button1.connect("clicked", self.do_formatblock_p)
         button1.show()
         self.toolbar1.add( button1)
@@ -923,7 +982,7 @@ class MainWindow:
         label1 = gtk.Label("")
         label1.set_markup("<big><big><b>H1</b></big></big>")
         button1 = gtk.ToolButton(label1, _("Heading 1"))
-        self.tooltips.set_tip(button1, _("Heading 1"))
+        button1.set_tooltip_text(_("Heading 1"))
         button1.connect("clicked", self.do_formatblock_h1)
         button1.show()
         self.toolbar1.add( button1)
@@ -931,7 +990,7 @@ class MainWindow:
         label1 = gtk.Label("")
         label1.set_markup("<big><b>H2</b></big>")
         button1 = gtk.ToolButton(label1, _("Heading 2"))
-        self.tooltips.set_tip(button1, _("Heading 2"))
+        button1.set_tooltip_text(_("Heading 2"))
         button1.connect("clicked", self.do_formatblock_h2)
         button1.show()
         self.toolbar1.add( button1)
@@ -941,7 +1000,7 @@ class MainWindow:
         label1 = gtk.Label("")
         label1.set_markup("<b>H3</b>")
         button1 = gtk.MenuToolButton(label1, _("Heading 3"))
-        self.tooltips.set_tip(button1, _("Heading 3"))
+        button1.set_tooltip_text(_("Heading 3"))
         button1.set_arrow_tooltip_markup(_("Style"))
         button1.connect("clicked", self.do_formatblock_h3)
         button1.show()
@@ -999,6 +1058,14 @@ class MainWindow:
 
         menu_style.append(menuitem_separator6)
 
+        div1 = gtk.ImageMenuItem(_("Di_v"))
+        div1.show()
+        div1.connect("activate", self.do_formatblock_div)
+
+        img = gtk.image_new_from_icon_name('stock_tools-hyphenation', gtk.ICON_SIZE_MENU)
+        div1.set_image(img)
+        menu_style.append(div1)
+
         address1 = gtk.ImageMenuItem(_("A_ddress"))
         address1.show()
         address1.connect("activate", self.do_formatblock_address)
@@ -1039,18 +1106,18 @@ class MainWindow:
         menu_format = gtk.Menu()
         menu_format.append(gtk.TearoffMenuItem())
 
-        menuitem_underline = gtk.ImageMenuItem("gtk-underline")
-        menuitem_underline.show()
-        menuitem_underline.connect("activate", self.do_underline)
-
-        menu_format.append(menuitem_underline)
-
         menuitem_italic = gtk.ImageMenuItem("gtk-italic")
         menuitem_italic.show()
         menuitem_italic.connect("activate", self.do_italic)
 
         menu_format.append(menuitem_italic)
 
+        menuitem_underline = gtk.ImageMenuItem("gtk-underline")
+        menuitem_underline.show()
+        menuitem_underline.connect("activate", self.do_underline)
+
+        menu_format.append(menuitem_underline)
+
         menuitem_strikethrough = gtk.ImageMenuItem("gtk-strikethrough")
         menuitem_strikethrough.show()
         menuitem_strikethrough.connect("activate", self.do_strikethrough)
@@ -1063,7 +1130,7 @@ class MainWindow:
 
         toolbutton_bold = gtk.MenuToolButton(gtk.STOCK_BOLD)
         toolbutton_bold.set_label(_("Bold"))
-        self.tooltips.set_tip(toolbutton_bold, _("Bold"))
+        toolbutton_bold.set_tooltip_text(_("Bold"))
         toolbutton_bold.show()
         toolbutton_bold.set_stock_id(gtk.STOCK_BOLD)
         toolbutton_bold.connect("clicked", self.on_bold)
@@ -1074,7 +1141,7 @@ class MainWindow:
         toolbutton_hilitecolor = gtk.MenuToolButton("")
         toolbutton_hilitecolor.set_icon_name("stock_text_color_hilight")
         toolbutton_hilitecolor.set_label(_("Highlight"))
-        self.tooltips.set_tip(toolbutton_hilitecolor, _("Highlight"))
+        toolbutton_hilitecolor.set_tooltip_text(_("Highlight"))
         toolbutton_hilitecolor.set_arrow_tooltip_markup(_("Select hilitecolor"))
         toolbutton_hilitecolor.set_menu(gtk.Menu())
         toolbutton_hilitecolor.show()
@@ -1089,7 +1156,7 @@ class MainWindow:
         button1 = gtk.ToolButton()
         button1.set_icon_name("gtk-clear")
         button1.set_label(_("Clear format"))
-        self.tooltips.set_tip(button1, _("Clear format"))
+        button1.set_tooltip_text(_("Clear format"))
         button1.show()
         button1.connect("clicked", self.do_removeformat)
         self.toolbar1.add(button1)
@@ -1097,7 +1164,7 @@ class MainWindow:
         ### 字体菜单按钮
         #toolbutton_font = gtk.MenuToolButton("gtk-select-font")
         #toolbutton_font.set_label(_("Font"))
-        #self.tooltips.set_tip(toolbutton_font, _("Font"))
+        #toolbutton_font.set_tooltip_text(_("Font"))
         #toolbutton_font.show()
         #toolbutton_font.set_menu(self.fontname_menu)
 
@@ -1135,7 +1202,7 @@ class MainWindow:
         self.notebox.set_tab_pos(2) # 0, 1, 2, 3 -> left, top, right, bottom
         self.notebox.set_border_width(0)
         #self.notebox.popup_enable()
-        self.notebox.set_homogeneous_tabs(0)
+        self.notebox.set_property('homogeneous', 0)
         self.notebox.unset_flags(gtk.CAN_FOCUS)
         self.notebox.set_scrollable(True)
         self.notebox.connect("switch-page", self.on_mdi_switch_page)
@@ -1185,7 +1252,7 @@ class MainWindow:
         self.findbox.pack_start(self.entry_searchtext)
 
         button1 = gtk.Button()
-        self.tooltips.set_tip(button1, _("Find Previous"))
+        button1.set_tooltip_text(_("Find Previous"))
         button1.show()
         button1.set_relief(gtk.RELIEF_NONE)
         button1.connect("clicked", self.do_find_text_backward)
@@ -1202,7 +1269,7 @@ class MainWindow:
         img.set_from_stock("gtk-find", 4)
         img.show()
         button_search_text.set_image(img)
-        self.tooltips.set_tip(button_search_text, _("Find Next"))
+        button_search_text.set_tooltip_text(_("Find Next"))
         button_search_text.show()
         button_search_text.set_relief(gtk.RELIEF_NONE)
         button_search_text.connect("clicked", self.do_find_text)
@@ -1220,7 +1287,7 @@ class MainWindow:
         self.findbox.pack_start(self.entry_replace_text)
 
         button_replace_text = gtk.Button()
-        self.tooltips.set_tip(button_replace_text, _("Replace"))
+        button_replace_text.set_tooltip_text(_("Replace"))
         button_replace_text.show()
         button_replace_text.set_relief(gtk.RELIEF_NONE)
         button_replace_text.connect("clicked", self.do_replace_text)
@@ -1250,7 +1317,7 @@ class MainWindow:
         #self.findbox.pack_start(gtk.VSeparator(), False, False, 0)
 
         button2 = gtk.Button()
-        self.tooltips.set_tip(button2, _("Replace All"))
+        button2.set_tooltip_text(_("Replace All"))
         button2.set_label(_("ReplaceAll"))
         button2.show()
         button2.set_relief(gtk.RELIEF_NONE)
@@ -1305,6 +1372,13 @@ class MainWindow:
         menu.show_all()
         return menu
 
+    def on_accel_connect_group(self, accel_group, acceleratable, keyval, modifier):
+        ## 按 Alt-1, Alt-2... 切换标签页
+        ## gtk.gdk.keyval_from_name('1') 为 49
+        num = keyval - 49
+        self.notebox.set_current_page(num)
+        return
+
     def on_mdi_menu(self, widget, event, editbox=None, *args):
         #-print self, widget, event, editbox, args
         if event.button == 3:
@@ -1472,6 +1546,8 @@ class MainWindow:
 
         editbox.connect("button-press-event", lambda *i: True) ## 中止鼠标按钮事件向上传递
 
+        gobject.idle_add(proc_webkit_color, edit, linkview)
+
         return editbox
 
 
@@ -2023,6 +2099,12 @@ class MainWindow:
         self.edit.do_insertorderedlist()
         pass
 
+    def do_formatblock_div(self, *args):
+        #-print 'do_formatblock_address:'
+        self.window.present()
+        self.edit.do_formatblock_address()
+        pass
+
     def do_formatblock_address(self, *args):
         #-print 'do_formatblock_address:'
         self.window.present()
@@ -2181,6 +2263,12 @@ class MainWindow:
         self.edit.do_justifycenter()
         pass
 
+    def do_justifyfull(self, *args):
+        #-print 'do_justify:'
+        self.window.present()
+        self.edit.do_justifyfull()
+        pass
+
     def do_justifyright(self, *args):
         #-print 'do_justifyright:'
         self.window.present()
@@ -2383,6 +2471,10 @@ def main():
     if settings.get_property( 'gtk-icon-theme-name' ) == 'hicolor':
         settings.set_property( 'gtk-icon-theme-name', 'Tango')
         pass
+    ## 处理额外图标路径
+    icon_theme = gtk.icon_theme_get_default()
+    icon_dir = os.path.dirname(__file__) + '/icons'
+    icon_theme.append_search_path(icon_dir)
     ##
     gtk.gdk.threads_enter()
     gtk.main()
