@@ -5,7 +5,7 @@
 # License: LGPLv3+
 # Last modified:
 
-__version__ = '0.3.0'
+__version__ = '0.5.1'
 
 import gtk, gobject
 import webkit
@@ -17,7 +17,18 @@ import re
 import docfilter
 
 try: import i18n
-except: import gettext.gettext as _
+except: from gettext import gettext as _
+
+def format_html(html):
+    '''给 html 添加适当换行
+    '''
+    html = re.sub('\n?\ ?<(address|blockquote|center|dir|div|dl|fieldset|form|h1|h2|h3|h4|h5|h6|hr|isindex|menu|noframes|noscript|ol|p|pre|table|ul|dd|dt|frameset|li|tbody|td|tfoot|th|thead|tr)([^>]*?)>', '\n<\\1\\2>', html)
+    html = re.sub('</(address|blockquote|center|dir|div|dl|fieldset|form|h1|h2|h3|h4|h5|h6|hr|isindex|menu|noframes|noscript|ol|p|pre|table|ul|dd|dt|frameset|li|tbody|td|tfoot|th|thead|tr)([^>]*?)>\ ?\n?', '</\\1\\2>\n', html)
+    html = re.sub('\n?<(img|hr|br)([^>]*?)>\n?', '\n<\\1\\2>\n', html)
+    ## 对于 pre，合并相邻 <pre>，将 <pre> 内的 <br> 转为 "\n"
+    html = re.sub('\n?</pre>\s*<pre>\n?', '', html)
+    html = re.sub('<pre>[^\0]*?</pre>', lambda m: re.sub('\n?<br>\n?', '\\n', m.group()), html)
+    return html
 
 def stastr(stri):
     '''处理字符串的  '   "
@@ -93,7 +104,8 @@ def menu_find_with_stock(menu, stock):
 BLANKHTML='''<!DOCTYPE html>
 <html>
 <head>
-  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+  <meta charset="utf-8" />
+  <meta http-equiv="X-UA-Compatible" content="IE=edge" />
   <meta name="generator" content="GWrite (WYSIWYG editor)" />
   <title></title>
   <style>
@@ -105,9 +117,11 @@ img{
 }
 h1, h2, h3, h4, h5, h6 {
     color: #7DA721;
+    font-weight: bold;
 }
 p{
     text-indent: 2em;
+    text-align: justify;
 }
 blockquote{
     background-color:#EEFFFF;
@@ -163,6 +177,16 @@ class WebKitEdit(webkit.WebView):
         self.connect("new-window-policy-decision-requested", self.on_new_window_policy_decision_requested)
         self.connect_after("populate-popup", self.populate_popup)
         self.connect("script-prompt", self.on_script_prompt)
+
+        ## 允许跨域 XMLHttpRequest，以便 base64 内联图片
+        settings = self.get_settings()
+        settings.set_property('enable-xss-auditor', False)
+        settings.set_property('enable-universal-access-from-file-uris', True)
+        settings.set_property('enable-file-access-from-file-uris', True)
+        settings.set_property('enable-page-cache', True)
+        settings.set_property('javascript-can-access-clipboard', True)
+        settings.set_property('tab-key-cycles-through-elements', False)
+
         ##
         pass
 
@@ -203,6 +227,7 @@ class WebKitEdit(webkit.WebView):
             self.execute_script('guesstitle();')
             self.do_image_base64()
             html = self.ctx().EvaluateScript('document.documentElement.innerHTML')
+            html = format_html(html)
             return '<!DOCTYPE html>\n<html>\n%s\n</html>\n' % html
         else:
             text = self.eval('''
@@ -212,6 +237,8 @@ class WebKitEdit(webkit.WebView):
                 text = document.body.textContent;
                 text;''')
             text = re.sub('\\s*</body>', '\n</body>', text)
+            text = text.replace(' \n', '\n')
+            text = format_html(text)
             return text
 
     def get_section(self, *args):
@@ -742,13 +769,13 @@ class WebKitEdit(webkit.WebView):
         '''剪切
         '''
         #print 'WebKitEdit.do_cut:'
-        self.execute_script(' document.execCommand("cut", false, false); ')
+        self.cut_clipboard()
 
     def do_copy(self, *args):
         '''复制
         '''
         #print 'WebKitEdit.do_copy:'
-        self.execute_script(' document.execCommand("copy", false, false); ')
+        self.copy_clipboard()
 
     def do_paste(self, *args):
         '''粘贴
@@ -933,6 +960,7 @@ class WebKitEdit(webkit.WebView):
 
     def do_formatblock_h1(self, *args):
         '''<h1> 样式
+        @NOTE: 将一行字设为标题后，回车出现的是 <div> 而非 <p>。需要让他换行后使用 <p>。考虑在下一行添加一个 <p>
         '''
         #print 'WebKitEdit.do_formatblock_h1:'
         return self.eval(''' 
@@ -1007,6 +1035,14 @@ class WebKitEdit(webkit.WebView):
                 document.execCommand("insertorderedlist", false, null); ''')
         pass
 
+    def do_formatblock_div(self, *args):
+        ''' DIV 样式
+        '''
+        #print 'WebKitEdit.formatblock_addres:'
+        return self.eval('''
+                document.execCommand("div", false, "address"); ''')
+        pass
+
     def do_formatblock_address(self, *args):
         '''地址样式
         '''
@@ -1185,10 +1221,18 @@ class WebKitEdit(webkit.WebView):
         '''居中
         '''
         #print 'WebKitEdit.do_justifycenter:'
-        self.execute_script(''' 
+        self.execute_script('''
                 document.execCommand("justifycenter", false, null); ''')
         pass
 
+    def do_justifyfull(self, *args):
+        '''分散对齐
+        '''
+        #print 'WebKitEdit.do_justifycenter:'
+        self.execute_script(''' 
+                document.execCommand("justifyfull", false, null); ''')
+        pass
+
     def do_justifyright(self, *args):
         '''右对齐
         '''
